from utils import *
from gfxmap import *
from player import Player

from pygsear.Path import VelocityPath, LinePath
from pygsear.Game import Game
from pygsear.Widget import ImageButton, Score
from pygsear.Drawable import String, SpriteGroup, Image, MultiImage
from pygsear.Event import KEYDOWN_Event
from pygsear.locals import DGREEN, DRED, YELLOW, BLACK, DBLUE
from pygame.locals import K_LEFT, K_RIGHT, K_UP, K_DOWN, K_r, K_ESCAPE, K_SPACE


LEVELS = ['stor-level1.map',
          'off-level1.map']

class MyGame(Game):
    def initialize(self):
        self.my_events = self.addEventGroup()
        self.window.set_title("Pyweek4 - The only way is up")
        self.start_screen()

    def start_screen(self, ev=None):
        self.clean_up()
        dirname = 'data/images/startmenu/'
        
        #Start Game Button
        button_start = ImageButton(filename=dirname+'start.png',
                                   callback=self.start_game)
#                                   callback=self.outro)
        button_start.set_position((562,404))
        self.sprites.add(button_start)
        self.my_events.add(button_start.events)
                                   
        #Quit Game Button
        button_quit = ImageButton(filename=dirname+'quit.png',
                                  callback=self.quit_game)
        button_quit.set_position((562,488))
        self.sprites.add(button_quit)
        self.my_events.add(button_quit.events)

        #add events to game
        self.events.add(self.my_events)
        self.set_background(filename=dirname+'titlescreen.png')
        self.update()
        
    def start_game(self, ev=None):
        """Starts the first level"""
        self.clean_up()
        self.intro()
        
        self.mylayer = self.addLayer()
        self.levelnr = 0
        self.mapsprites = self.addGroup(layer=self.mylayer)
        self.set_background(color=BLACK)
        self.keyscore = Score(position=(750,0),
                              text="Keys:",
                              digits=2,
                              fontSize=20,
                              color=YELLOW)
        self.sprites.add(self.keyscore, level=5)

        self.johnny = Player(imgdir='data/images/johnny') #bad gyu 1
        self.evilguy = Player(imgdir='data/images/evilguy') #bad guy 2
        self.player = Player(imgdir='data/images/guy_with_cat')
        self.sprites.add(self.player, level=1)
        self.load_level(mapname=LEVELS[0])
        self.sprites.add(self.mapsprites, level=0)
        
        self.my_events.add(KEYDOWN_Event(key=K_UP, 
                                         callback=self.move_player))
        self.my_events.add(KEYDOWN_Event(key=K_DOWN, 
                                         callback=self.move_player))
        self.my_events.add(KEYDOWN_Event(key=K_LEFT, 
                                         callback=self.move_player))
        self.my_events.add(KEYDOWN_Event(key=K_RIGHT,
                                         callback=self.move_player))
        self.my_events.add(KEYDOWN_Event(key=K_r, 
                                         callback=self.reload_level))
        self.my_events.add(KEYDOWN_Event(key=K_ESCAPE,
                                         callback=self.start_screen))
        self.events.add(self.my_events)
        self.update()
        
    def move_player(self, ev=None, chara=None):
        def move_crate(tile, dest_row, dest_col):
            desttile = self.level.tiles[dest_row][dest_col]
            kind = desttile.kind
            if desttile.walkable and kind!='%' and kind!='*' and kind!='S':
                tile.move_crate(desttile)
                return True
            return False
        
        def take_key(tile):
            #player gets key, keyfield removed
            SND_key.play()
            chara.keys += 1
            tile.remove_key()
            self.reload_score()

        def use_key(tile):
            SND_door.play()
            chara.keys -= 1
            self.reload_score()
            tile.open_keydoor()

        def use_button(tile):
            SND_button.play()
            tile.switch_button()
            doorpos = convert_coords(self.level.buttondoor)
            col = doorpos[0]
            row = doorpos[1]
            door = self.level.tiles[row][col]
            door.open_buttondoor()

        player_move = False
        upstairs = False
        he_moved = False
        if not chara:           #who shall be moved?
            chara = self.player
            player_move = True
            key = ev.key
        else:
            key = ev
            if chara == 'ai0':
                chara = self.johnny
            else:
                chara = self.evilguy
                
        col = chara.coords[0]
        row = chara.coords[1]

        #bleh. moving around.... o.O
        if key == K_UP:
            chara.flip(imageName='guy_up.png')
            tile = self.level.tiles[row-1][col]
            if tile.walkable:
                chara.move_up()
                he_moved = True
            elif tile.kind is 'C': #Crate?
                if move_crate(tile=tile, dest_row=row-2, dest_col=col):
                    chara.move_up()
                    he_moved = True
        elif key == K_DOWN:
            chara.flip(imageName='guy_down.png')
            tile = self.level.tiles[row+1][col]
            if tile.walkable:
                chara.move_down()
                he_moved = True
            elif tile.kind is 'C': #Crate?
                if move_crate(tile=tile, dest_row=row+2, dest_col=col):
                    chara.move_down()
                    he_moved = True
        elif key == K_LEFT:
            chara.flip(imageName='guy_left.png')
            tile = self.level.tiles[row][col-1]
            if tile.walkable:
                chara.move_left()
                he_moved = True
            elif tile.kind is 'C': #Crate?
                if move_crate(tile=tile, dest_row=row, dest_col=col-2):
                   chara.move_left()
                   he_moved = True
        elif key == K_RIGHT:
            chara.flip(imageName='guy_right.png')
            tile = self.level.tiles[row][col+1]
            if tile.walkable:
                chara.move_right()
                he_moved = True
            elif tile.kind is 'C': #Crate?
                if move_crate(tile=tile, dest_row=row, dest_col=col+2):
                    chara.move_right()
                    he_moved = True

        if tile.kind is 'K':
            take_key(tile)
        elif tile.kind is '%' and not tile.walkable:
            if chara.keys > 0:
                use_key(tile)
        elif tile.kind is 'B' and player_move:
            use_button(tile)
        elif tile.kind is 'E' and player_move:
            upstairs = True
        elif tile.kind is 'T':
            tile.kind = 'G'
            self.guards_coming()

        if player_move and self.guards:
            #move guards after player has moved
            for i in AI_Checklist1:
                if self.move_player(ev=i[self.johnny.looking], chara='ai0'):
                    break
            for i in AI_Checklist2:
                if self.move_player(ev=i[self.evilguy.looking], chara='ai1'):
                    break

        if he_moved and not player_move:
            self.level.tiles[row][col].walkable = True
            tile.walkable = False
        if self.player.collide(self.johnny) or self.player.collide(self.evilguy):
            self.reload_level()
        if upstairs:
            self.next_level()
        return he_moved

    def guards_coming(self):
        SND_breakup.play()
        self.guards = True
        self.step()
        s = convert_coords(self.level.start)
        self.level.tiles[s[1]][s[0]].image.flip(imageName='stairs1.png')
        
        self.johnny.set_position(self.level.start)
        self.johnny.coords = convert_coords(self.level.start)
        
        pos = (self.level.start[0], self.level.start[1])
        self.evilguy.set_position(pos)
        self.evilguy.coords=convert_coords(pos)
        self.sprites.add(self.johnny, level=1)
        self.sprites.add(self.evilguy, level=1)
            
    def load_level(self, mapname):
        self.johnny.set_position((0,0))
        self.evilguy.set_position((0,0))
        self.sprites.remove(self.mapsprites)
        self.mapsprites.empty()
        self.johnny.kill()
        self.evilguy.kill()
        self.guards = False
        self.level = GfxMap(filename=mapname)
        for y in self.level.tiles:
            for x in y:
                self.mapsprites.add(x.image)
        self.sprites.add(self.mapsprites, level=0)
        self.player.keys = 0
        self.player.set_position(self.level.start)
        self.player.coords = convert_coords(self.level.start)
        self.reload_score()
        self.sprites.clear()
        self.sprites.draw()
        if self.level.trigger:
            self.enter_anim()

    def reload_level(self, ev=None):
        self.sprites.remove(self.johnny, self.evilguy)
        self.load_level(mapname=LEVELS[self.levelnr])
        
    def enter_anim(self):
        self.player.flip(imageName='guy_down.png')
        self.step()
        start = convert_coords(self.level.start)
        self.step()
        self.step()
        self.step()
        self.player.move_down()
        self.step()
        self.step()
        self.player.flip(imageName='guy_up.png')
        self.step()
        self.step()
        self.level.tiles[start[1]][start[0]].image.flip(imageName='stairs1_2.png')
        SND_close_stairs.play()

    def exit_anim(self):
        #play footsteps
        SND_stairs_up.play()
        self.waitFor(timeout=1000)
        self.window.clear()

    def next_level(self):
        self.exit_anim()
        self.levelnr += 1
        if self.levelnr == LEVELS.__len__():
            self.outro()
        elif self.levelnr==1: #show catscene
            self.catscene()
            self.load_level(mapname=LEVELS[self.levelnr])
        else:
            self.load_level(mapname=LEVELS[self.levelnr])

    def reload_score(self):
        self.keyscore.set_points(self.player.keys)
        self.keyscore.updateScore()

    def gameOver(self):
        self.waitFor(timeout=100)
        self.clean_up()
        msg = String(message='Game Over', fontSize=60, color=DRED, bgcolor=BLACK)
        msg2 = String(message='Press [ESC]',fontSize=30,color=DBLUE,bgcolor=BLACK)
        msg.center()
        msg2.center(y=500)
        self.sprites.add(msg)
        self.sprites.add(msg2)
        self.my_events.add(KEYDOWN_Event(key=K_ESCAPE,callback=self.start_screen))
        self.events.add(self.my_events)

    def clean_up(self, ev=None):
        self.sprites.kill()
        self.sprites.empty()
        self.events.remove(self.my_events)
        self.my_events.empty()
        self.update()

    def quit_game(self, ev=None):
        self._quit()

    def outro(self, ev=None):
#FIXME!!!!!!!!!!!!!11111111111
        self.clean_up()
        self.set_background(color=BLACK)
        self.waitFor(timeout=2000)

        msg = ' Press [ESC] to exit to menu '
        msgsprite = String(message=msg,
                           fontSize=25,
                           color=BLACK,
                           bgcolor=(128,255,255))
        msgsprite.center(y=10)
        self.sprites.add(msgsprite, level=2)

        dirname='data/images/outro/'
        self.set_background(filename=dirname+'scenery.png')

        pos = (550, 250)
        jonny = Image(filename=dirname+'jonny_left.png')
        jonny.set_position(pos)
        self.sprites.add(jonny, level=2)

        pos = (570, 200)
        evilguy = Image(filename=dirname+'evilguy.png')
        evilguy.set_position(pos)
        self.sprites.add(evilguy, level=2)

        pos = (576,182)
        door = Image(filename=dirname+'door2.png')
        door.set_position(pos)
        self.sprites.add(door, level=1)

        pos = (208,178)
        ballon = Image(filename=dirname+'ballon2.png')
        ballon.set_position(pos)
        self.sprites.add(ballon, level=2)
        self.step()
        self.step()
        self.step()
        ballon.set_path(VelocityPath(startLocation=pos,vx=-1.5,vy=-1))
        self.step()
        self.step()
        self.step()

    def diashow(self, ev=None, imglist=[]):
        msg = 'Press [SPACE] to continue'
        msgsprite = String(message=msg,
                           fontSize=25,
                           color=YELLOW,
                           bgcolor=BLACK)
        msgsprite.center(y=-100)
        for i in imglist:
            self.set_background(filename=i)
            msgsprite.udraw()
            self.waitFor(key=K_SPACE)
        self.set_background(color=BLACK)

    def catscene(self, ev=None):
        dirname='data/images/intro/'
        imglist = [dirname+'catscene01.png']
        self.diashow(imglist=imglist)
        
    def intro(self, ev=None):
        dirname='data/images/intro/'
        imglist=[dirname+'intro01.png',
                 dirname+'intro02.png',
                 dirname+'intro03.png',
                 dirname+'intro04.png']
        self.diashow(imglist=imglist)
        
#disable pygsear splashscreen for quicker testing
    def splash_screen(self):
        pass
    def splash_screen_poof(self):
        pass
    def configure(self, ev=None):
        self.start_screen()

def main():
    g = MyGame()
    g.mainloop()

#remove me, please ;)
if __name__ == '__main__':
    main()
